@@ -11,7 +11,6 @@
  */
 
 #include "gmp_agent_groups.h"
-#include "manage.h"
 #include "manage_acl.h"
 
 #undef G_LOG_DOMAIN
@@ -23,20 +22,21 @@
 /* GET_AGENT_GROUPS */
 
 /**
- * @struct get_agent_groups_t
- * @brief data for <get_agent_groups> command
- *
+ * @brief data for `<get_agent_groups>` command
  */
 typedef struct
 {
-    get_data_t get;
+  get_data_t get;  ///< Get args.
 } get_agent_groups_t;
 
 /**
- * @brief Parser <get_agent_groups> callback data.
+ * @brief Parser `<get_agent_groups>` callback data.
  */
 static get_agent_groups_t get_agent_groups_data;
 
+/**
+ * @brief Reset command data.
+ */
 static void
 get_agent_groups_reset ()
 {
@@ -180,7 +180,7 @@ get_agent_groups_run (gmp_parser_t *gmp_parser, GError **error)
  */
 typedef struct
 {
-  context_data_t *context;
+  context_data_t *context;     ///< XML parser context.
 } create_agent_group_t;
 
 /**
@@ -465,6 +465,12 @@ create_agent_group_run (gmp_parser_t *gmp_parser, GError **error)
       log_event_fail ("agent_group", "Agent Group", NULL, "created");
       break;
 
+    case AGENT_GROUP_RESP_GROUP_NAME_EXISTS:
+      SEND_TO_CLIENT_OR_FAIL (
+        XML_ERROR_SYNTAX ("create_agent_group", "Agent Group name exists"));
+      log_event_fail ("agent_group", "Agent Group", NULL, "created");
+      break;
+
     case AGENT_GROUP_RESP_INTERNAL_ERROR:
     default:
         SEND_TO_CLIENT_OR_FAIL (XML_INTERNAL_ERROR ("create_agent_group"));
@@ -487,19 +493,21 @@ create_agent_group_run (gmp_parser_t *gmp_parser, GError **error)
 /* MODIFY_AGENT_GROUP */
 
 /**
- * @struct modify_agent_group_data_t
- * @brief data for <modify_agent_group> command
+ * @brief data for `<modify_agent_group>` command
  */
 typedef struct
 {
-    context_data_t *context;
+  context_data_t *context;     ///< XML parser context.
 } modify_agent_group_data_t;
 
 /**
- * @brief Parser <modify_agent_group> callback data.
+ * @brief Parser `<modify_agent_group>` callback data.
  */
 static modify_agent_group_data_t modify_agent_group_data;
 
+/**
+ * @brief Reset command data.
+ */
 static void
 modify_agent_group_reset ()
 {
@@ -514,7 +522,7 @@ modify_agent_group_reset ()
 }
 
 /**
- * @brief Start the element in the <modify_agent_group> command.
+ * @brief Start the element in the `<modify_agent_group>` command.
  *
  * @param[in] gmp_parser       Active GMP parser instance.
  * @param[in] name             Name of the XML element being parsed.
@@ -534,7 +542,7 @@ modify_agent_group_element_start (gmp_parser_t *gmp_parser,
 }
 
 /**
- * @brief Initialize the <modify_agent_group> GMP command.
+ * @brief Initialize the ``<modify_agent_group>`` GMP command.
  *
  * @param[in] gmp_parser        Active GMP parser instance.
  * @param[in] attribute_names   Null-terminated array of attribute names.
@@ -565,7 +573,7 @@ modify_agent_group_element_text (const gchar *text, gsize text_len)
 }
 
 /**
- * @brief End the XML element within the <modify_agent_group> command.
+ * @brief End the XML element within the `<modify_agent_group>` command.
  *
  * @param[in] gmp_parser  Active GMP parser instance
  * @param[in] error       The errors, if any
@@ -587,7 +595,7 @@ modify_agent_group_element_end (gmp_parser_t *gmp_parser, GError **error,
 }
 
 /**
- * @brief Execute the <modify_agent_group> GMP command.
+ * @brief Execute the `<modify_agent_group>` GMP command.
  *
  * @param[in] gmp_parser  Active GMP parser instance.
  * @param[in] error       the errors, if any.
@@ -751,6 +759,12 @@ modify_agent_group_run (gmp_parser_t *gmp_parser, GError **error)
         log_event_fail ("agent_group", "Agent Group", NULL, "modified");
         break;
 
+      case AGENT_GROUP_RESP_GROUP_NAME_EXISTS:
+        SEND_TO_CLIENT_OR_FAIL (
+          XML_ERROR_SYNTAX ("modify_agent_group", "Agent Group name exists"));
+        log_event_fail ("agent_group", "Agent Group", NULL, "modified");
+        break;
+
       case AGENT_GROUP_RESP_INTERNAL_ERROR:
       default:
         SEND_TO_CLIENT_OR_FAIL (XML_INTERNAL_ERROR ("modify_agent_group"));
