@@ -1,19 +1,6 @@
 /* Copyright (C) 2009-2022 Greenbone AG
  *
  * SPDX-License-Identifier: AGPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -54,7 +41,7 @@
 #include "manage_http_scanner.h"
 #endif
 
-#if OPENVASD
+#if ENABLE_OPENVASD
 #include <gvm/openvasd/openvasd.h>
 #endif
 
@@ -141,6 +128,9 @@ typedef struct
 typedef int (*manage_connection_forker_t) (gvm_connection_t * conn,
                                            const gchar* uuid);
 
+void
+init_manage_funcs ();
+
 int
 init_manage (GSList*, const db_conn_info_t *, int, int, int, int,
              manage_connection_forker_t, int);
@@ -151,6 +141,9 @@ init_manage_helper (GSList *, const db_conn_info_t *, int, int);
 void
 init_manage_process (const db_conn_info_t*);
 
+void
+reinit_manage_process ();
+
 void
 cleanup_manage_process (gboolean);
 
@@ -160,12 +153,25 @@ manage_cleanup_process_error (int);
 void
 manage_reset_currents ();
 
+void
+manage_session_init (const char *);
+
 /* Commands. */
 
 #define MAX_LOCK_RETRIES 16
 
 #define MAX_DATABASE_CONNECTIONS_DEFAULT 50
 
+/**
+ * @brief Maximum number of retries to lock a table.
+ */
+#define MAX_TABLE_LOCK_RETRIES_DEFAULT 128
+
+/**
+ * @brief Timeout for trying to acquire a table lock in milliseconds.
+ */
+#define LOCK_TIMEOUT 500
+
 #define MAX_REPORT_PROCESSING_DEFAULT 30
 
 /* Certificate and key management. */
@@ -260,15 +266,12 @@ current_encryption_key_uid (gboolean);
 void
 set_current_encryption_key_uid (const char *new_uid);
 
+int
+validate_sort_field (const gchar*, const gchar*);
+
 void
 manage_session_set_timezone (const char *);
 
-void
-manage_transaction_start ();
-
-void
-manage_transaction_stop (gboolean);
-
 
 /* Task macros and structures. */
 
@@ -355,6 +358,20 @@ typedef enum scanner_type
   SCANNER_TYPE_MAX = 11,
 } scanner_type_t;
 
+/**
+ * @brief Scanner feature status.
+ *
+ * These numbers are indicating whether the required feature
+ * for a given scanner type is available.
+ */
+typedef enum
+{
+ SCANNER_FEATURE_OK = 0,
+ SCANNER_FEATURE_OPENVASD_DISABLED = 1,
+ SCANNER_FEATURE_AGENTS_DISABLED = 2,
+ SCANNER_FEATURE_CONTAINER_DISABLED = 3
+} scanner_feature_status_t;
+
 int
 scanner_type_valid (scanner_type_t);
 
@@ -364,42 +381,24 @@ scanner_type_supports_unix_sockets (scanner_type_t);
 scanner_type_t
 get_scanner_type (scanner_t);
 
+scanner_feature_status_t
+check_scanner_feature (scanner_type_t);
+
 scanner_type_t
 get_scanner_type_by_uuid (const char *);
 
 
 /* Resources. */
 
-int
-manage_resource_name (const char *, const char *, char **);
-
-int
-manage_trash_resource_name (const char *, const char *, char **);
-
-int
-resource_count (const char *, const get_data_t *);
-
 int
 resource_id_exists (const char *, const char *);
 
 int
 trash_id_exists (const char *, const char *);
 
-gboolean
-find_resource (const char*, const char*, resource_t*);
-
-gboolean
-find_resource_no_acl (const char*, const char*, resource_t*);
-
 int
 delete_resource (const char *, const char *, int);
 
-int
-resource_id_deprecated (const char *, const char *);
-
-void
-set_resource_id_deprecated (const char *, const char *, gboolean);
-
 
 /* Events and Alerts. */
 
@@ -545,9 +544,6 @@ task_iterator_last_report (iterator_t *);
 report_t
 task_iterator_current_report (iterator_t *);
 
-const char *
-task_iterator_hosts_ordering (iterator_t *);
-
 scanner_t
 task_iterator_scanner (iterator_t *);
 
@@ -590,9 +586,6 @@ task_name (task_t);
 char*
 task_comment (task_t);
 
-char*
-task_hosts_ordering (task_t);
-
 scanner_t
 task_scanner (task_t);
 
@@ -656,10 +649,10 @@ task_oci_image_target_in_trash (task_t);
 void
 set_task_oci_image_target (task_t, oci_image_target_t);
 
-#endif /* ENABLE_CONTAINER_SCANNING */
-
 void
-set_task_hosts_ordering (task_t, const char *);
+clear_task_asset_preferences (task_t);
+
+#endif /* ENABLE_CONTAINER_SCANNING */
 
 void
 set_task_scanner (task_t, scanner_t);
@@ -748,9 +741,6 @@ task_schedule_in_trash (task_t);
 time_t
 task_schedule_next_time_uuid (const gchar *);
 
-int
-task_schedule_next_time (task_t);
-
 int
 task_debugs_size (task_t);
 
@@ -814,9 +804,6 @@ append_to_task_comment (task_t, const char*, int);
 void
 add_task_description_line (task_t, const char*, size_t);
 
-void
-set_scan_ports (report_t, const char*, unsigned int, unsigned int);
-
 void
 append_task_open_port (task_t task, const char *, const char*);
 
@@ -829,9 +816,8 @@ manage_task_remove_file (const gchar *, const char *);
 int
 modify_task (const gchar *, const gchar *, const gchar *, const gchar *,
              const gchar *, const gchar *, const gchar *, array_t *,
-             const gchar *, array_t *, const gchar *, const gchar *,
-             array_t *, const gchar *, const gchar *, const gchar *,
-             gchar **, gchar **);
+             const gchar *, array_t *, const gchar *, const gchar *, array_t *,
+             const gchar *, const gchar *, gchar **, gchar **);
 
 void
 init_config_file_iterator (iterator_t*, const char*, const char*);
@@ -1027,12 +1013,12 @@ report_severity (report_t, int, int);
 int
 report_host_count (report_t);
 
-int
-report_result_host_count (report_t, int);
-
 char *
 report_finished_hosts_str (report_t);
 
+char *
+report_finished_container_images_str (report_t);
+
 gboolean
 find_report_with_permission (const char *, report_t *, const char *);
 
@@ -1190,11 +1176,11 @@ set_scan_end_time_epoch (report_t, time_t);
 void
 set_scan_host_start_time_ctime (report_t, const char*, const char*);
 
-int
-scan_host_end_time (report_t, const char*);
+void
+set_scan_host_start_time_isotime (report_t, const char*, const char*);
 
 void
-set_scan_host_end_time (report_t, const char*, const char*);
+set_scan_host_end_time_isotime (report_t, const char*, const char* );
 
 void
 set_scan_host_end_time_ctime (report_t, const char*, const char*);
@@ -1373,6 +1359,21 @@ result_iterator_dfn_certs (iterator_t*);
 const char *
 result_iterator_compliance (iterator_t*);
 
+const char *
+result_iterator_oci_image_name (iterator_t*);
+
+const char *
+result_iterator_oci_image_digest (iterator_t*);
+
+const char *
+result_iterator_oci_image_registry (iterator_t*);
+
+const char *
+result_iterator_oci_image_path (iterator_t*);
+
+const char *
+result_iterator_oci_image_short_name (iterator_t*);
+
 const char *
 result_iterator_delta_state (iterator_t*);
 
@@ -1614,185 +1615,15 @@ prognosis_iterator_description (iterator_t*);
  */
 #define MANAGE_USER_MAX_HOSTS 16777216
 
-int
-manage_max_hosts ();
-
-int
-manage_count_hosts (const char *, const char *);
-
-gboolean
-find_target_with_permission (const char *, target_t *, const char *);
-
-int
-create_target (const char*, const char*, const char*, const char*, const char*,
-               const char *, const char*, credential_t, credential_t,
-               const char *,
-               credential_t, credential_t, credential_t, credential_t,
-               const char *, const char *, const char *, const char *,
-               target_t*);
-
-int
-copy_target (const char*, const char*, const char *, target_t*);
-
-int
-modify_target (const char*, const char*, const char*, const char*, const char*,
-               const char*, const char*, const char*, const char*, const char*,
-               const char*, const char*, const char*, const char*, const char*,
-               const char*, const char*);
-
-int
-delete_target (const char*, int);
-
 int
 target_count (const get_data_t *);
 
-void
-init_user_target_iterator (iterator_t*, target_t);
-
-void
-init_target_iterator_one (iterator_t*, target_t);
-
-int
-init_target_iterator (iterator_t*, get_data_t *);
-
-const char*
-target_iterator_hosts (iterator_t*);
-
-const char*
-target_iterator_exclude_hosts (iterator_t*);
-
-const char*
-target_iterator_reverse_lookup_only (iterator_t*);
-
-const char*
-target_iterator_reverse_lookup_unify (iterator_t*);
-
-const char*
-target_iterator_comment (iterator_t*);
-
-int
-target_iterator_ssh_credential (iterator_t*);
-
-const char*
-target_iterator_ssh_port (iterator_t*);
-
-int
-target_iterator_smb_credential (iterator_t*);
-
-int
-target_iterator_esxi_credential (iterator_t*);
-
-int
-target_iterator_snmp_credential (iterator_t*);
-
-int
-target_iterator_ssh_elevate_credential (iterator_t*);
-
-int
-target_iterator_krb5_credential (iterator_t*);
-
-int
-target_iterator_ssh_trash (iterator_t*);
-
-int
-target_iterator_smb_trash (iterator_t*);
-
-int
-target_iterator_esxi_trash (iterator_t*);
-
-int
-target_iterator_snmp_trash (iterator_t*);
-
-int
-target_iterator_ssh_elevate_trash (iterator_t*);
-
-int
-target_iterator_krb5_trash (iterator_t*);
-
-const char*
-target_iterator_allow_simultaneous_ips (iterator_t*);
-
-const char*
-target_iterator_port_list_uuid (iterator_t*);
-
-const char*
-target_iterator_port_list_name (iterator_t*);
-
-int
-target_iterator_port_list_trash (iterator_t*);
-
-const char*
-target_iterator_alive_tests (iterator_t*);
-
-char*
-target_uuid (target_t);
-
-char*
-trash_target_uuid (target_t);
-
-char*
-target_name (target_t);
-
-char*
-trash_target_name (target_t);
-
-int
-trash_target_readable (target_t);
-
-char*
-target_hosts (target_t);
-
-char*
-target_exclude_hosts (target_t);
-
-char*
-target_reverse_lookup_only (target_t);
-
-char*
-target_reverse_lookup_unify (target_t);
-
-char*
-target_allow_simultaneous_ips (target_t);
-
 char*
 target_port_range (target_t);
 
-char*
-target_ssh_port (target_t);
-
-int
-target_in_use (target_t);
-
-int
-trash_target_in_use (target_t);
-
-int
-target_writable (target_t);
-
-int
-trash_target_writable (target_t);
-
 char*
 target_ssh_credential_name (const char *);
 
-void
-init_target_task_iterator (iterator_t*, target_t);
-
-const char*
-target_task_iterator_name (iterator_t*);
-
-const char*
-target_task_iterator_uuid (iterator_t*);
-
-int
-target_task_iterator_readable (iterator_t*);
-
-credential_t
-target_credential (target_t, const char*);
-
-int
-target_login_port (target_t, const char*);
-
 
 /* Configs.
  *
@@ -1807,9 +1638,6 @@ modify_task_check_config_scanner (task_t, const char *, const char *);
 
 /* NVT's. */
 
-char *
-manage_nvt_name (nvt_t);
-
 char *
 nvt_name (const char *);
 
@@ -1928,6 +1756,9 @@ nvt_iterator_max_epss_severity (iterator_t*);
 gboolean
 nvt_iterator_has_max_epss_severity (iterator_t*);
 
+int
+nvt_iterator_discovery (iterator_t*);
+
 char*
 nvt_default_timeout (const char *);
 
@@ -2101,6 +1932,7 @@ create_credential (const char*, const char*, const char*, const char*,
                    const char*, const char*, const char*, const char*,
                    const char*, const char*, const char*, const char*,
                    array_t*,    const char*, const char*, const char*,
+                   const char*, const char*, const char*, const char*,
                    credential_t*);
 
 int
@@ -2111,7 +1943,8 @@ int
 modify_credential (const char*, const char*, const char*, const char*,
                    const char*, const char*, const char*, const char*,
                    const char*, const char*, const char*, const char*,
-                   const char*, array_t*, const char*, const char*);
+                   const char*, array_t*, const char*, const char*,
+                   const char*, const char*, const char*, const char*);
 
 int
 delete_credential (const char *, int);
@@ -2158,6 +1991,22 @@ credential_iterator_kdc (iterator_t*);
 const char*
 credential_iterator_realm (iterator_t*);
 
+#if ENABLE_CREDENTIAL_STORES
+
+const char*
+credential_iterator_credential_store_uuid (iterator_t*);
+
+const char*
+credential_iterator_vault_id (iterator_t*);
+
+const char*
+credential_iterator_host_identifier (iterator_t*);
+
+const char*
+credential_iterator_privacy_host_identifier (iterator_t*);
+
+#endif
+
 const char*
 credential_iterator_private_key (iterator_t*);
 
@@ -2483,20 +2332,23 @@ manage_get_scanners (GSList *, const db_conn_info_t *);
 
 
 typedef enum {
-  CREATE_SCANNER_INTERNAL_ERROR = -1,     ///< Internal error
-  CREATE_SCANNER_SUCCESS = 0,             ///< Success
-  CREATE_SCANNER_ALREADY_EXISTS,          ///< Scanner already exists
-  CREATE_SCANNER_MISSING_TYPE,            ///< Missing type
-  CREATE_SCANNER_MISSING_HOST,            ///< Missing host
-  CREATE_SCANNER_CREDENTIAL_NOT_FOUND,    ///< Credential not found
-  CREATE_SCANNER_CREDENTIAL_NOT_CC,       ///< Credential must have type "cc"
-  CREATE_SCANNER_INVALID_TYPE,            ///< Invalid type
-  CREATE_SCANNER_INVALID_PORT,            ///< Invalid port
-  CREATE_SCANNER_INVALID_HOST,            ///< Invalid host
-  CREATE_SCANNER_INVALID_RELAY_PORT,      ///< Invalid relay port
-  CREATE_SCANNER_INVALID_RELAY_HOST,      ///< Invalid relay host
-  CREATE_SCANNER_UNIX_SOCKET_UNSUPPORTED, ///< Type doesn't support UNIX sockets
-  CREATE_SCANNER_PERMISSION_DENIED = 99   ///< Permission denied
+  CREATE_SCANNER_INTERNAL_ERROR = -1,         ///< Internal error
+  CREATE_SCANNER_SUCCESS = 0,                 ///< Success
+  CREATE_SCANNER_ALREADY_EXISTS,              ///< Scanner already exists
+  CREATE_SCANNER_MISSING_TYPE,                ///< Missing type
+  CREATE_SCANNER_MISSING_HOST,                ///< Missing host
+  CREATE_SCANNER_CREDENTIAL_NOT_FOUND,        ///< Credential not found
+  CREATE_SCANNER_CREDENTIAL_NOT_CC,           ///< Credential must have type "cc"
+  CREATE_SCANNER_INVALID_TYPE,                ///< Invalid type
+  CREATE_SCANNER_INVALID_PORT,                ///< Invalid port
+  CREATE_SCANNER_INVALID_HOST,                ///< Invalid host
+  CREATE_SCANNER_INVALID_RELAY_PORT,          ///< Invalid relay port
+  CREATE_SCANNER_INVALID_RELAY_HOST,          ///< Invalid relay host
+  CREATE_SCANNER_UNIX_SOCKET_UNSUPPORTED,     ///< Type doesn't support UNIX sockets
+  CREATE_SCANNER_OPENVASD_DISABLED,           ///< openvasd feature is disabled
+  CREATE_SCANNER_AGENT_DISABLED,              ///< Agent feature is disabled
+  CREATE_SCANNER_CONTAINER_SCANNING_DISABLED, ///< Container scanning feature is disabled
+  CREATE_SCANNER_PERMISSION_DENIED = 99       ///< Permission denied
 } create_scanner_return_t;
 
 create_scanner_return_t
@@ -2508,20 +2360,23 @@ int
 copy_scanner (const char*, const char*, const char *, scanner_t *);
 
 typedef enum {
-  MODIFY_SCANNER_INTERNAL_ERROR = -1,     ///< Internal error
-  MODIFY_SCANNER_SUCCESS = 0,             ///< Success
-  MODIFY_SCANNER_ALREADY_EXISTS,          ///< Scanner already exists
-  MODIFY_SCANNER_MISSING_ID,              ///< Missing scanner id
-  MODIFY_SCANNER_NOT_FOUND,               ///< Scanner not found
-  MODIFY_SCANNER_CREDENTIAL_NOT_FOUND,    ///< Credential not found
-  MODIFY_SCANNER_CREDENTIAL_NOT_CC,       ///< Credential must have type "cc"
-  MODIFY_SCANNER_INVALID_TYPE,            ///< Invalid type
-  MODIFY_SCANNER_INVALID_PORT,            ///< Invalid port
-  MODIFY_SCANNER_INVALID_HOST,            ///< Invalid host
-  MODIFY_SCANNER_INVALID_RELAY_PORT,      ///< Invalid relay port
-  MODIFY_SCANNER_INVALID_RELAY_HOST,      ///< Invalid relay host
-  MODIFY_SCANNER_UNIX_SOCKET_UNSUPPORTED, ///< Type doesn't support UNIX sockets
-  MODIFY_SCANNER_PERMISSION_DENIED = 99   ///< Permission denied
+  MODIFY_SCANNER_INTERNAL_ERROR = -1,         ///< Internal error
+  MODIFY_SCANNER_SUCCESS = 0,                 ///< Success
+  MODIFY_SCANNER_ALREADY_EXISTS,              ///< Scanner already exists
+  MODIFY_SCANNER_MISSING_ID,                  ///< Missing scanner id
+  MODIFY_SCANNER_NOT_FOUND,                   ///< Scanner not found
+  MODIFY_SCANNER_CREDENTIAL_NOT_FOUND,        ///< Credential not found
+  MODIFY_SCANNER_CREDENTIAL_NOT_CC,           ///< Credential must have type "cc"
+  MODIFY_SCANNER_INVALID_TYPE,                ///< Invalid type
+  MODIFY_SCANNER_INVALID_PORT,                ///< Invalid port
+  MODIFY_SCANNER_INVALID_HOST,                ///< Invalid host
+  MODIFY_SCANNER_INVALID_RELAY_PORT,          ///< Invalid relay port
+  MODIFY_SCANNER_INVALID_RELAY_HOST,          ///< Invalid relay host
+  MODIFY_SCANNER_UNIX_SOCKET_UNSUPPORTED,     ///< Type doesn't support UNIX sockets
+  MODIFY_SCANNER_OPENVASD_DISABLED,           ///< openvasd feature is disabled
+  MODIFY_SCANNER_AGENT_DISABLED,              ///< Agent feature is disabled
+  MODIFY_SCANNER_CONTAINER_SCANNING_DISABLED, ///< Container scanning feature is disabled
+  MODIFY_SCANNER_PERMISSION_DENIED = 99       ///< Permission denied
 } modify_scanner_return_t;
 
 modify_scanner_return_t
@@ -2574,12 +2429,6 @@ scanner_key_pub (scanner_t);
 char *
 scanner_key_priv (scanner_t);
 
-char*
-scanner_login (scanner_t);
-
-char*
-scanner_password (scanner_t);
-
 int
 scanner_count (const get_data_t *);
 
@@ -2637,6 +2486,9 @@ scanner_task_iterator_uuid (iterator_t *);
 const char*
 scanner_task_iterator_name (iterator_t *);
 
+const char *
+scanner_task_iterator_usage_type (iterator_t *);
+
 int
 scanner_task_iterator_readable (iterator_t *);
 
@@ -2646,12 +2498,6 @@ scanner_name (scanner_t);
 char *
 scanner_uuid (scanner_t);
 
-char *
-trash_scanner_name (scanner_t);
-
-char *
-trash_scanner_uuid (scanner_t);
-
 int
 osp_get_version_from_iterator (iterator_t *, char **, char **, char **, char **,
                                char **, char **);
@@ -2674,9 +2520,6 @@ get_relay_mapper_path ();
 void
 set_relay_mapper_path (const char *);
 
-gboolean
-relay_supports_scanner_type (const char *, int, scanner_type_t);
-
 int
 slave_get_relay (const char *,
                  int,
@@ -2690,6 +2533,10 @@ int
 slave_relay_connection (gvm_connection_t *, gvm_connection_t *);
 
 /* Scheduling. */
+/**
+* @brief Seconds of a day
+*/
+#define SECONDS_PER_DAY 86400
 
 /**
  * @brief Seconds between calls to manage_schedule.
@@ -2712,6 +2559,18 @@ slave_relay_connection (gvm_connection_t *, gvm_connection_t *);
  */
 #define SCHEDULE_TIMEOUT_DEFAULT 60
 
+#if ENABLE_AGENTS
+  /**
+  * @brief Seconds between calls to fork_agents_sync.
+  */
+  #define AGENT_SYNC_SCHEDULE_PERIOD 300 /* every 5 minutes */
+#endif
+
+/**
+ * @brief Seconds between calls to manage_asset_snapshot_delete_stale.
+ */
+#define ASSET_SNAPSHOT_STALE_DELETE_PERIOD 3600 /* every hour */
+
 gboolean
 find_schedule_with_permission (const char*, schedule_t*, const char*);
 
@@ -2818,169 +2677,6 @@ get_schedule_timeout ();
 void
 set_schedule_timeout (int);
 
-
-/* Groups. */
-
-int
-init_group_iterator (iterator_t *, get_data_t *);
-
-int
-copy_group (const char *, const char *, const char *, group_t *);
-
-int
-create_group (const char *, const char *, const char *, int, group_t *);
-
-int
-delete_group (const char *, int);
-
-char*
-group_uuid (group_t);
-
-gchar *
-group_users (group_t);
-
-int
-trash_group_in_use (group_t);
-
-int
-group_in_use (group_t);
-
-int
-trash_group_writable (group_t);
-
-int
-group_writable (group_t);
-
-int
-group_count (const get_data_t*);
-
-int
-modify_group (const char *, const char *, const char *, const char *);
-
-
-/* Permissions. */
-
-int
-create_permission (const char *, const char *, const char *, const char *,
-                   const char *, const char *, permission_t *);
-
-int
-copy_permission (const char*, const char *, permission_t *);
-
-char*
-permission_uuid (permission_t);
-
-int
-permission_is_admin (const char *);
-
-int
-permission_in_use (permission_t);
-
-int
-trash_permission_in_use (permission_t);
-
-int
-permission_writable (permission_t);
-
-int
-trash_permission_writable (permission_t);
-
-int
-permission_count (const get_data_t *);
-
-int
-init_permission_iterator (iterator_t*, get_data_t *);
-
-const char*
-permission_iterator_resource_type (iterator_t*);
-
-const char*
-permission_iterator_resource_uuid (iterator_t*);
-
-const char*
-permission_iterator_resource_name (iterator_t*);
-
-int
-permission_iterator_resource_in_trash (iterator_t*);
-
-int
-permission_iterator_resource_orphan (iterator_t*);
-
-int
-permission_iterator_resource_readable (iterator_t*);
-
-const char*
-permission_iterator_subject_type (iterator_t*);
-
-const char*
-permission_iterator_subject_uuid (iterator_t*);
-
-const char*
-permission_iterator_subject_name (iterator_t*);
-
-int
-permission_iterator_subject_in_trash (iterator_t*);
-
-int
-permission_iterator_subject_readable (iterator_t*);
-
-int
-delete_permission (const char*, int);
-
-int
-modify_permission (const char *, const char *, const char *, const char *,
-                   const char *, const char *, const char *);
-
-/* Permission caching */
-
-void
-delete_permissions_cache_for_resource (const char*, resource_t);
-
-void
-delete_permissions_cache_for_user (user_t);
-
-
-/* Roles. */
-
-int
-manage_get_roles (GSList *, const db_conn_info_t *, int);
-
-int
-init_role_iterator (iterator_t *, get_data_t *);
-
-int
-copy_role (const char *, const char *, const char *, role_t *);
-
-int
-create_role (const char *, const char *, const char *, role_t *);
-
-int
-delete_role (const char *, int);
-
-char*
-role_uuid (role_t);
-
-gchar *
-role_users (role_t);
-
-int
-trash_role_in_use (role_t);
-
-int
-role_in_use (role_t);
-
-int
-trash_role_writable (role_t);
-
-int
-role_writable (role_t);
-
-int
-role_count (const get_data_t*);
-
-int
-modify_role (const char *, const char *, const char *, const char *);
-
 
 /* Schema. */
 
@@ -3010,10 +2706,8 @@ info_name_count (const gchar *, const gchar *);
 int
 manage_scap_loaded ();
 
-#if FEED_VT_METADATA == 1
 int
 manage_nvts_loaded ();
-#endif
 
 const char *
 manage_scap_update_time ();
@@ -3219,124 +2913,8 @@ int
 manage_modify_setting (GSList *, const db_conn_info_t *, const gchar *,
                        const gchar *, const char *);
 
-char *
-manage_default_ca_cert ();
-
 
-/* Users. */
-
-gboolean
-find_user_by_name_with_permission (const char *, user_t *, const char *);
-
-int
-manage_create_user (GSList *, const db_conn_info_t *, const gchar *,
-                    const gchar *, const gchar *);
-
-int
-manage_delete_user (GSList *, const db_conn_info_t *, const gchar *,
-                    const gchar *);
-
-int
-manage_get_users (GSList *, const db_conn_info_t *, const gchar *, int);
-
-int
-manage_set_password (GSList *, const db_conn_info_t *, const gchar *,
-                     const gchar *);
-
-gchar *
-manage_user_hash (const gchar *);
-
-gchar *
-manage_user_uuid (const gchar *, auth_method_t);
-
-int
-manage_user_exists (const gchar *, auth_method_t);
-
-int
-copy_user (const char*, const char*, const char*, user_t*);
-
-gchar *
-keyfile_to_auth_conf_settings_xml (const gchar *);
-
-int
-init_user_iterator (iterator_t*, get_data_t*);
-
-const char*
-user_iterator_role (iterator_t*);
-
-const char*
-user_iterator_method (iterator_t*);
-
-const char*
-user_iterator_hosts (iterator_t*);
-
-int
-user_iterator_hosts_allow (iterator_t*);
-
-void
-init_user_group_iterator (iterator_t *, user_t);
-
-const char*
-user_group_iterator_uuid (iterator_t*);
-
-const char*
-user_group_iterator_name (iterator_t*);
-
-int
-user_group_iterator_readable (iterator_t*);
-
-void
-init_user_role_iterator (iterator_t *, user_t);
-
-const char*
-user_role_iterator_uuid (iterator_t*);
-
-const char*
-user_role_iterator_name (iterator_t*);
-
-int
-user_role_iterator_readable (iterator_t*);
-
-int
-create_user (const gchar *, const gchar *, const gchar *, const gchar *,
-             int, const array_t *, array_t *, gchar **,
-             array_t *, gchar **, gchar **, user_t *, int);
-
-int
-delete_user (const char *, const char *, int, const char*, const char*);
-
-int
-modify_user (const gchar *, gchar **, const gchar *, const gchar *,
-             const gchar*, const gchar *, int,
-             const array_t *, array_t *, gchar **, array_t *, gchar **,
-             gchar **);
-
-int
-user_in_use (user_t);
-
-int
-trash_user_in_use (user_t);
-
-int
-user_writable (user_t);
-
-int
-trash_user_writable (user_t);
-
-int
-user_count (const get_data_t*);
-
-gchar*
-user_name (const char *);
-
-char*
-user_uuid (user_t);
-
-gchar*
-user_hosts (const char *);
-
-int
-user_hosts_allow (const char *);
+/* Vulns. */
 
 int
 init_vuln_iterator (iterator_t*, const get_data_t*);
@@ -3568,9 +3146,15 @@ set_max_concurrent_scan_updates (int);
 int
 get_max_database_connections ();
 
+int
+get_max_table_lock_retries ();
+
 void
 set_max_database_connections (int);
 
+void
+set_max_table_lock_retries (int);
+
 int
 get_max_concurrent_report_processing ();
 
@@ -3607,18 +3191,6 @@ feed_lockfile_unlock (lockfile_t *);
 int
 gvm_migrate_secinfo (int);
 
-gboolean
-gvm_sync_script_perform_selftest (const gchar *, gchar **);
-
-gboolean
-gvm_get_sync_script_identification (const gchar *, gchar **, int);
-
-gboolean
-gvm_get_sync_script_description (const gchar *, gchar **);
-
-gboolean
-gvm_get_sync_script_feed_version (const gchar *, gchar **);
-
 int
 nvts_feed_info (gchar **, gchar **, gchar **, gchar **);
 
@@ -3631,6 +3203,24 @@ manage_rebuild (GSList *, const db_conn_info_t *);
 int
 manage_dump_vt_verification (GSList *, const db_conn_info_t *);
 
+/* Asset Snapshots */
+
+#define ASSET_SNAPSHOT_MANAGED_POLICY_DAY 90
+
+void
+manage_asset_snapshot_delete_stale (int);
+
+int
+manage_dump_asset_snapshot_counts (GSList *, const db_conn_info_t *);
+
+
+/* Timezone info. */
+array_t *
+manage_get_timezones ();
+
+gboolean
+manage_timezone_supported (const char *);
+
 
 /* Wizards. */
 
@@ -3648,11 +3238,14 @@ xml_escape_text_truncated (const char *, size_t, const char *);
 int
 column_is_timestamp (const char*);
 
-char*
-type_columns (const char *);
+column_t*
+type_select_columns (const char *);
 
-char*
-type_trash_columns (const char *);
+column_t*
+type_where_columns (const char *);
+
+const char**
+type_filter_columns (const char *);
 
 gboolean
 manage_migrate_needs_timezone (GSList *, const db_conn_info_t *);
