@@ -16,6 +16,7 @@
 #if ENABLE_AGENTS
 #include "manage_agents.h"
 #include "manage_sql_agents.h"
+#include "manage_sql_resources.h"
 
 #include <assert.h>
 
@@ -184,6 +185,11 @@ convert_agent_control_list_to_agent_data_list (
       dest->operating_system = g_strdup (src->operating_system);
       dest->architecture = g_strdup (src->architecture);
       dest->update_to_latest = src->update_to_latest;
+      dest->last_updater_heartbeat = src->last_updater_heartbeat;
+      dest->agent_update_available = src->agent_update_available;
+      dest->updater_update_available = src->updater_update_available;
+      dest->latest_agent_version = g_strdup (src->latest_agent_version);
+      dest->latest_updater_version = g_strdup (src->latest_updater_version);
 
       dest->scanner = scanner;
 
@@ -478,6 +484,12 @@ agent_data_free (agent_data_t data)
   if (data->comment)
     g_free (data->comment);
 
+  if (data->latest_agent_version)
+    g_free (data->latest_agent_version);
+
+  if (data->latest_updater_version)
+    g_free (data->latest_updater_version);
+
   agent_ip_data_list_free (data->ip_addresses);
 
   g_free (data);
@@ -636,6 +648,8 @@ get_agents_by_scanner_and_uuids (scanner_t scanner, agent_uuid_list_t uuid_list,
     }
   if (count != uuid_list->count)
     {
+      g_debug ("%s: agent iterator agent count: %d, user agent list count: %d",
+               __func__, count, uuid_list->count);
       cleanup_iterator (&iterator);
       return AGENT_RESPONSE_AGENT_NOT_FOUND;
     }
@@ -831,4 +845,43 @@ delete_and_resync_agents (agent_uuid_list_t agent_uuids)
   return AGENT_RESPONSE_SUCCESS;
 }
 
-#endif // ENABLE_AGENTS
\ No newline at end of file
+const gchar *
+agent_response_to_string (agent_response_t code)
+{
+  switch (code)
+    {
+    case AGENT_RESPONSE_SUCCESS:
+      return "Success";
+    case AGENT_RESPONSE_NO_AGENTS_PROVIDED:
+      return "No agent UUIDs provided";
+    case AGENT_RESPONSE_SCANNER_LOOKUP_FAILED:
+      return "Scanner lookup failed";
+    case AGENT_RESPONSE_AGENT_SCANNER_MISMATCH:
+      return "Agent list count mismatch (not same scanner)";
+    case AGENT_RESPONSE_CONNECTOR_CREATION_FAILED:
+      return "Failed to create connector";
+    case AGENT_RESPONSE_CONTROLLER_UPDATE_FAILED:
+      return "Failed to update agents";
+    case AGENT_RESPONSE_CONTROLLER_DELETE_FAILED:
+      return "Failed to delete agents";
+    case AGENT_RESPONSE_SYNC_FAILED:
+      return "Failed during sync";
+    case AGENT_RESPONSE_INVALID_ARGUMENT:
+      return "Invalid argument";
+    case AGENT_RESPONSE_INVALID_AGENT_OWNER:
+      return "Invalid agent owner UUID";
+    case AGENT_RESPONSE_AGENT_NOT_FOUND:
+      return "Agent not found";
+    case AGENT_RESPONSE_INTERNAL_ERROR:
+      return "Internal error";
+    case AGENT_RESPONSE_IN_USE_ERROR:
+      return "Agent is used by an Agent Group";
+    case AGENT_RESPONSE_CONTROLLER_UPDATE_REJECTED:
+      return "Agent update validation error";
+    default:
+      return "Unknown agent error";
+    }
+}
+
+#endif // ENABLE_AGENTS
+
