@@ -1,25 +1,12 @@
 /* Copyright (C) 2025 Greenbone AG
  *
  * SPDX-License-Identifier: AGPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _GVMD_MANAGE_ALERTS_H
 #define _GVMD_MANAGE_ALERTS_H
 
-#include "iterator.h"
+#include "manage_resources_types.h"
 #include "manage_events.h"
 #include "manage_get.h"
 #include "manage_tasks.h"
@@ -27,8 +14,6 @@
 
 #include <glib.h>
 
-typedef resource_t alert_t;
-
 int
 get_max_email_attachment_size ();
 
@@ -217,4 +202,8 @@ alert_task_iterator_readable (iterator_t*);
 int
 manage_check_alerts (GSList *, const db_conn_info_t *);
 
+int
+trigger (alert_t, task_t, report_t, event_t, const void *, alert_method_t,
+         alert_condition_t, const get_data_t *, int, int, gchar **);
+
 #endif /* not _GVMD_MANAGE_ALERTS_H */
