@@ -1,25 +1,12 @@
 /* Copyright (C) 2019-2022 Greenbone AG
  *
  * SPDX-License-Identifier: AGPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _GVMD_MANAGE_CONFIGS_H
 #define _GVMD_MANAGE_CONFIGS_H
 
-#include "manage_resources.h"
+#include "manage_resources_types.h"
 #include "manage_get.h"
 #include "manage_preferences.h"
 
@@ -169,4 +156,10 @@ should_sync_configs ();
 gboolean
 scanner_type_requires_config (int scanner_type);
 
+void
+get_nvt_oids_from_config (config_t, GSList **);
+
+void
+get_nvt_oids_from_config_uuid (const gchar *, GSList **);
+
 #endif /* not _GVMD_MANAGE_CONFIGS_H */
