@@ -12,10 +12,11 @@
 #if ENABLE_CONTAINER_SCANNING
 
 #include "debug_utils.h"
+#include "manage_assets.h"
 #include "manage_container_image_scanner.h"
+#include "manage_runtime_flags.h"
 #include "manage_sql.h"
 #include "manage_sql_oci_image_targets.h"
-#include "manage.h"
 #include "manage_openvas.h"
 #include "manage_osp.h"
 #include <sys/types.h>
@@ -98,7 +99,7 @@ container_image_target_credential (oci_image_target_t target)
         "username",
         credential_iterator_login (&iter)
       );
-      
+
       container_image_credential_set_auth_data (
         container_image_credential,
         "password",
@@ -120,7 +121,7 @@ container_image_target_credential (oci_image_target_t target)
  *                               report_aux.
  */
 static void
-add_container_image_scan_result (http_scanner_result_t res, 
+add_container_image_scan_result (http_scanner_result_t res,
                                  gpointer *results_aux)
 {
 
@@ -128,41 +129,80 @@ add_container_image_scan_result (http_scanner_result_t res,
   result_t result;
   char *type, *host, *hostname, *test_id;
   char *port = NULL, *desc = NULL;
-  char *nvt_id = NULL, *severity_str = NULL;
-  int qod_int;
 
-  type = convert_http_scanner_type_to_osp_type (res->type);
+  type = res->type;
   test_id = res->oid;
-  host = res->ip_address;
+
+  host = extract_sha256_digest_if_found (res->ip_address);
   hostname = res->hostname;
   port = res->port;
-
-  nvt_id = g_strdup (test_id);
-  severity_str = nvt_severity (test_id, type);
   desc = res->message;
-  qod_int = get_http_scanner_nvti_qod (test_id);
 
-  char *hash_value;
-  if (!check_http_scanner_result_exists (rep_aux->report, rep_aux->task, res,
-                                         &hash_value, rep_aux->hash_results))
+  if (host)
+    manage_report_host_add (rep_aux->report, host, 0, 0);
+
+  if (strcmp (type, "host_detail") == 0)
     {
-      result = make_osp_result (rep_aux->task,
-                                host ?: "",
-                                hostname ?: "",
-                                nvt_id ?: "",
-                                type ?: "",
-                                desc ?: "",
-                                port ?: "",
-                                severity_str ?: NULL,
-                                qod_int,
-                                NULL,
-                                hash_value);
-      g_array_append_val (rep_aux->results_array, result);
+      gchar *hash_value = NULL;
+      if (!check_host_detail_exists (rep_aux->report, host,
+                                     res->detail_source_type,
+                                     res->detail_source_name,
+                                     res->detail_source_description,
+                                     res->detail_name,
+                                     res->detail_value,
+                                     &hash_value,
+                                     rep_aux->hash_hostdetails))
+        {
+          insert_report_host_detail (rep_aux->report, host,
+                                     res->detail_source_type,
+                                     res->detail_source_name,
+                                     res->detail_source_description,
+                                     res->detail_name,
+                                     res->detail_value,
+                                     hash_value);
+        }
     }
+  else if (strcmp (type, "host_start") == 0)
+    {
+      set_scan_host_start_time_isotime (rep_aux->report, host, desc);
+    }
+  else if (strcmp (type, "host_end") == 0)
+    {
+      set_scan_host_end_time_isotime (rep_aux->report, host, desc);
+    }
+  else
+    {
+      char *nvt_id = NULL, *severity_str = NULL;
+      char *hash_value, *result_type;
+      int qod_int;
 
-  g_free (hash_value);
-  g_free (nvt_id);
-  g_free (type);
+      nvt_id = g_strdup (test_id);
+      result_type = convert_http_scanner_type_to_osp_type (type);
+      severity_str = nvt_severity (test_id, result_type);
+      qod_int = get_http_scanner_nvti_qod (test_id);
+
+      if (!check_http_scanner_result_exists (rep_aux->report, rep_aux->task,
+                                             res, &hash_value,
+                                             rep_aux->hash_results))
+        {
+          result = make_osp_result (rep_aux->task,
+                                    host ?: "",
+                                    hostname ?: "",
+                                    nvt_id ?: "",
+                                    result_type ?: "",
+                                    desc ?: "",
+                                    port ?: "",
+                                    severity_str ?: NULL,
+                                    qod_int,
+                                    NULL,
+                                    hash_value);
+          g_array_append_val (rep_aux->results_array, result);
+        }
+      g_free (hash_value);
+      g_free (severity_str);
+      g_free (result_type);
+      g_free (nvt_id);
+    }
 
   return;
 }
@@ -181,9 +221,9 @@ parse_container_image_scan_report (task_t task,
                                    time_t start_time,
                                    time_t end_time)
 {
-  gboolean has_results = FALSE;
   GArray *results_array = NULL;
   GHashTable *hashed_results = NULL;
+  GHashTable *hashed_host_details = NULL;
   struct report_aux *rep_aux;
 
   assert (task);
@@ -203,13 +243,15 @@ parse_container_image_scan_report (task_t task,
       sql_commit ();
       return;
     }
- 
+
   hashed_results = g_hash_table_new_full (g_str_hash,
                                           g_str_equal,
                                           g_free,
                                           NULL);
-
-  has_results = TRUE;
+  hashed_host_details = g_hash_table_new_full (g_str_hash,
+                                               g_str_equal,
+                                               g_free,
+                                               NULL);
 
   results_array = g_array_new(TRUE, TRUE, sizeof(result_t));
   rep_aux = g_malloc0 (sizeof (struct report_aux));
@@ -217,13 +259,13 @@ parse_container_image_scan_report (task_t task,
   rep_aux->task = task;
   rep_aux->results_array = results_array;
   rep_aux->hash_results = hashed_results;
-  rep_aux->hash_hostdetails = NULL;
+  rep_aux->hash_hostdetails = hashed_host_details;
 
-  g_slist_foreach(results, 
+  g_slist_foreach(results,
                   (GFunc) add_container_image_scan_result,
                   &rep_aux);
 
-  if (has_results)
+  if (results_array->len > 0)
     {
       sql ("UPDATE reports SET modification_time = m_now() WHERE id = %llu;",
            report);
@@ -231,10 +273,11 @@ parse_container_image_scan_report (task_t task,
     }
 
   sql_commit ();
-  if (results_array && has_results)
+  if (results_array && results_array->len > 0)
     g_array_free (results_array, TRUE);
 
   g_hash_table_destroy (hashed_results);
+  g_hash_table_destroy (hashed_host_details);
   g_free (rep_aux);
 }
 
@@ -290,14 +333,48 @@ prepare_container_image_scan_for_resume (task_t task, const char *scan_id,
 char *
 get_bool_string (const char *preference_value)
 {
-  if (strcmp (preference_value, "no") == 0)
+  if (strcmp (preference_value, "no") == 0
+      || strcmp (preference_value, "0") == 0)
     return g_strdup("false");
-  else if (strcmp (preference_value, "yes") == 0)
+  else if (strcmp (preference_value, "yes") == 0
+           || strcmp (preference_value, "1") == 0)
     return g_strdup("true");
   else
     return g_strdup (preference_value);
 }
 
+/*
+* @brief Get the value of a named scanner preference from a list of preferences.
+*
+* @param[in]  name        The name of the scanner preferences.
+* @param[in]  scan_prefs  The list of preferences.
+*
+* @return     The value of the preference.
+*/
+static gchar *
+get_preference_from_list (char *name, GSList *scan_prefs)
+{
+  gchar *value = NULL;
+  GSList *point = scan_prefs;
+
+  while (point)
+    {
+      http_scanner_param_t *param;
+      char *p_name;
+
+      param = point->data;
+      p_name = http_scanner_param_id (param);
+
+      if (p_name && strcmp (p_name, name) == 0)
+        {
+          value = g_strdup (http_scanner_param_default (param));
+          break;
+        }
+      point = g_slist_next (point);
+    }
+  return value;
+}
+
 /*
 * @brief Add container image scan preferences to the scanner options.
 *        Task preferences override default scanner preferences.
@@ -305,11 +382,16 @@ get_bool_string (const char *preference_value)
 * @param[in]  scanner_options  The scanner preferences table to add to.
 * @param[in]  task             The task.
 *
+* @return     0 on success, -1 if error.
 */
-void
-add_container_image_scan_preferences (GHashTable *scanner_options, task_t task)
+static int
+add_container_image_scan_preferences (http_scanner_connector_t connector,
+                                      GHashTable *scanner_options,
+                                      task_t task)
 {
+  GSList *scan_prefs = NULL;
   gchar *accept_invalid_certs, *registry_allow_insecure;
+  int err = 0;
 
   accept_invalid_certs = task_preference_value (task, "accept_invalid_certs");
 
@@ -323,8 +405,40 @@ add_container_image_scan_preferences (GHashTable *scanner_options, task_t task)
     g_hash_table_insert (scanner_options, g_strdup ("registry_allow_insecure"),
                          get_bool_string (registry_allow_insecure));
 
+
+  if (!accept_invalid_certs || !registry_allow_insecure)
+    err = http_scanner_parsed_scans_preferences (connector, &scan_prefs);
+
+  if (err < 0)
+    {
+      if (scan_prefs)
+        g_slist_free_full (scan_prefs, (GDestroyNotify) http_scanner_param_free);
+      g_free (accept_invalid_certs);
+      g_free (registry_allow_insecure);
+      return err;
+    };
+
+  if (scan_prefs && !accept_invalid_certs)
+    {
+      accept_invalid_certs = get_preference_from_list ("accept_invalid_certs", scan_prefs);
+      if (accept_invalid_certs)
+        g_hash_table_insert (scanner_options, g_strdup ("accept_invalid_certs"),
+                             get_bool_string (accept_invalid_certs));
+    }
+
+  if (scan_prefs && !registry_allow_insecure)
+    {
+      registry_allow_insecure = get_preference_from_list ("registry_allow_insecure", scan_prefs);
+      if (registry_allow_insecure)
+        g_hash_table_insert (scanner_options, g_strdup ("registry_allow_insecure"),
+                             get_bool_string (registry_allow_insecure));
+    }
+
+  g_slist_free_full (scan_prefs, (GDestroyNotify) http_scanner_param_free);
   g_free (accept_invalid_certs);
   g_free (registry_allow_insecure);
+
+  return err;
 }
 
 /**
@@ -349,15 +463,16 @@ launch_container_image_task (task_t task,
                              char **error)
 {
   http_scanner_connector_t connection;
-  char *oci_image_references_str;
+  char *oci_image_references_str, *finished_images_str, *exclude_images_str;
   container_image_target_t *container_image_target;
   container_image_credential_t *credential;
-  gchar *max_checks, *max_hosts, *hosts_ordering;
+  gchar *max_checks, *max_hosts;
   GHashTable *scanner_options;
   http_scanner_resp_t response;
   int ret;
 
   connection = NULL;
+  finished_images_str = NULL;
 
   /* Prepare the report */
   if (from)
@@ -367,13 +482,34 @@ launch_container_image_task (task_t task,
         return 0;
       else if (ret == -1)
         return -1;
+      finished_images_str = report_finished_container_images_str (global_current_report);
     }
 
   oci_image_references_str
     = oci_image_target_image_references (oci_image_target);
 
-  container_image_target 
-      = container_image_target_new (scan_id, oci_image_references_str);
+  exclude_images_str = oci_image_target_exclude_images (oci_image_target);
+
+  GString *target_exclude_images
+    = g_string_new (exclude_images_str ? exclude_images_str : "");
+
+  if (finished_images_str != NULL && strlen (finished_images_str) > 0)
+    {
+      g_string_append_printf (target_exclude_images,
+                              "%s%s",
+                              exclude_images_str
+                                && strlen (exclude_images_str) > 0 ? "," : "",
+                              finished_images_str);
+    }
+
+  container_image_target
+      = container_image_target_new (scan_id, oci_image_references_str,
+                                    target_exclude_images->str);
+
+  g_string_free (target_exclude_images, TRUE);
+  free (exclude_images_str);
+  free (finished_images_str);
+  free (oci_image_references_str);
 
   credential = container_image_target_credential (oci_image_target);
 
@@ -398,14 +534,6 @@ launch_container_image_task (task_t task,
   g_hash_table_insert (scanner_options, g_strdup ("max_hosts"),
                        max_hosts ? max_hosts : g_strdup (MAX_HOSTS_DEFAULT));
 
-  hosts_ordering = task_hosts_ordering (task);
-  if (hosts_ordering)
-    g_hash_table_insert (scanner_options, g_strdup ("hosts_ordering"),
-                         hosts_ordering);
-
-  /* Setup container image scanner preferences */
-  add_container_image_scan_preferences (scanner_options, task);
-
   connection = container_image_scanner_connect (task_scanner (task), scan_id);
   if (!connection)
     {
@@ -416,6 +544,18 @@ launch_container_image_task (task_t task,
       return -1;
     }
 
+  /* Setup container image scanner preferences */
+  if (add_container_image_scan_preferences (connection,
+                                            scanner_options,
+                                            task) < 0)
+    {
+      if (error)
+        *error = g_strdup ("Could not get scan preferences from scanner");
+      container_image_target_free (container_image_target);
+      g_hash_table_destroy (scanner_options);
+      return -1;
+    }
+
   gchar *scan_config = NULL;
   scan_config =
     container_image_build_scan_config_json (container_image_target,
@@ -470,7 +610,7 @@ handle_container_image_scan (task_t task,
   scanner_t scanner;
   http_scanner_connector_t connector;
   int ret;
-  
+
   scanner = task_scanner (task);
   connector = container_image_scanner_connect (scanner, scan_id);
 
@@ -598,6 +738,7 @@ fork_container_image_scan_handler (task_t task,
       set_task_run_status (task, TASK_STATUS_PROCESSING);
       set_report_scan_run_status (global_current_report,
                                   TASK_STATUS_PROCESSING);
+      asset_snapshots_container_image (global_current_report, task);
       set_task_run_status (task, TASK_STATUS_DONE);
       set_report_scan_run_status (global_current_report, TASK_STATUS_DONE);
     }
@@ -634,6 +775,11 @@ fork_container_image_scan_handler (task_t task,
 int
 run_container_image_task (task_t task, int from, char **report_id)
 {
+  if (!feature_enabled (FEATURE_ID_CONTAINER_SCANNING))
+    {
+      g_warning ("%s: container scanning runtime flag is disabled", __func__);
+      return -1;
+    }
   oci_image_target_t oci_image_target;
 
   oci_image_target = task_oci_image_target (task);
@@ -657,7 +803,7 @@ run_container_image_task (task_t task, int from, char **report_id)
     }
 
   if (fork_container_image_scan_handler (task,
-                                         oci_image_target, 
+                                         oci_image_target,
                                          from,
                                          report_id))
     {
@@ -677,6 +823,11 @@ run_container_image_task (task_t task, int from, char **report_id)
 int
 stop_container_image_task (task_t task)
 {
+  if (!feature_enabled (FEATURE_ID_CONTAINER_SCANNING))
+    {
+      g_warning ("%s: container scanning runtime flag is disabled", __func__);
+      return -1;
+    }
   int ret = 0;
   report_t scan_report;
   char *scan_id;
@@ -740,4 +891,4 @@ end_stop_openvasd:
   return ret;
 }
 
-#endif
\ No newline at end of file
+#endif
