@@ -14,6 +14,7 @@
 
 #include "manage_oci_image_targets.h"
 #include "manage_sql.h"
+#include "manage_sql_resources.h"
 
 #include <gvm/base/hosts.h>
 
@@ -27,7 +28,7 @@
  * @brief Find an OCI image target for a specific permission, given a UUID.
  *
  * @param[in]   uuid               UUID of target.
- * @param[out]  oci_image_target   OCI Image Target return, 
+ * @param[out]  oci_image_target   OCI Image Target return,
  *                                 0 if successfully failed to find target.
  * @param[in]   permission         Permission.
  *
@@ -173,9 +174,9 @@ valid_oci_url (const gchar *oci_url)
     }
 
   g_free (port);
-  
+
   const gchar *url_component_regex = "^[a-zA-Z0-9_\\-.]+$";
-  const gchar *url_component_with_tag_regex = "^[a-zA-Z0-9_\\-.:]+$";
+  const gchar *url_component_with_tag_regex = "^[a-zA-Z0-9_\\-.:@]+$";
 
   for (int i = 1; i < parts_len; i++)
     {
@@ -203,6 +204,70 @@ valid_oci_url (const gchar *oci_url)
   return 0;
 }
 
+/**
+ * @brief Clean an OCI images string.
+ *
+ * @param[in]  given_images  String describing images.
+ *
+ * @return Freshly allocated new images string. NULL if error.
+ *         Caller must free it.
+ */
+gchar*
+clean_images (const char *given_images)
+{
+  gchar **split, **point, *images, *start;
+
+  if (!given_images || !*given_images)
+    return NULL;
+
+  /* Treat newlines like commas. */
+  images = start = g_strdup (given_images);
+  while (*images)
+    {
+      if (*images == '\n') *images = ',';
+      images++;
+    }
+
+  split = g_strsplit (start, ",", -1);
+  g_free (start);
+  point = split;
+
+  GHashTable *seen = g_hash_table_new_full (g_str_hash,
+                                            g_str_equal,
+                                            g_free, NULL);
+  GPtrArray *clean_array = make_array ();
+
+  while (*point)
+    {
+      g_strstrip (*point);
+      if (**point != '\0')
+        {
+          gchar *key = g_strdup (*point);
+          if (!g_hash_table_contains (seen, key))
+            {
+              g_hash_table_add (seen, key);
+              array_add (clean_array, g_strdup(*point));
+            }
+          else
+            {
+              g_free (key);
+            }
+        }
+      point += 1;
+    }
+  array_terminate (clean_array);
+
+  gchar *clean = (clean_array->len == 0)
+    ? g_strdup ("")
+    : g_strjoinv (",", (gchar **) clean_array->pdata);
+
+  g_hash_table_destroy (seen);
+  array_free (clean_array);
+  g_strfreev (split);
+
+  return clean;
+}
+
 /**
  * @brief Validate an OCI image references string.
  *
@@ -215,7 +280,7 @@ int
 validate_oci_image_references (const char *image_refs_input,
                                gchar **error_message)
 {
-  if (!image_refs_input || !*image_refs_input)
+  if (!image_refs_input)
     return FALSE;
 
   gchar *input_copy = g_strdup (image_refs_input);
@@ -236,25 +301,13 @@ validate_oci_image_references (const char *image_refs_input,
           return FALSE;
         }
 
-      // reject whitespace
-      for (const gchar *c = entry; *c; ++c)
-        {
-          if (g_ascii_isspace (*c))
-            {
-              g_strfreev (parts);
-              if (error_message)
-                *error_message = 
-                  g_strdup_printf ("Whitespace found in OCI image URL");
-              return FALSE;
-            }
-        }
 
       // validate OCI URL format
       if (valid_oci_url (entry) == -1)
         {
           g_strfreev (parts);
           if (error_message)
-            *error_message = 
+            *error_message =
               g_strdup_printf ("Invalid OCI image URL");
           return FALSE;
         }
