@@ -1,32 +1,21 @@
 /* Copyright (C) 2020-2022 Greenbone AG
  *
  * SPDX-License-Identifier: AGPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
- * @file manage_sql_port_lists.c
- * @brief GVM management layer: Port list SQL
+ * @file
+ * @brief GVM management layer: Port lists.
  *
- * The Port List SQL for the GVM management layer.
+ * General functions for managing Port Lists.
  */
 
 #include "manage_port_lists.h"
 #include "gmp_port_lists.h"
-#include "manage.h"
+#include "manage_sql.h"
+#include "manage_sql_permissions.h"
 #include "manage_sql_port_lists.h"
+#include "manage_users.h"
 #include "utils.h"
 
 #include <gvm/util/fileutils.h>
