@@ -8,8 +8,8 @@
  * @brief Headers for Greenbone Vulnerability Manager scan handler.
  */
 
-#ifndef _GVMD_SCAN_HANDLER_H
-#define _GVMD_SCAN_HANDLER_H
+#ifndef _GVMD_MANAGE_SCAN_HANDLER_H
+#define _GVMD_MANAGE_SCAN_HANDLER_H
 
 #include "manage_scan_queue.h"
 #include <glib.h>
@@ -17,4 +17,4 @@
 int
 fork_scan_handler (const char *, report_t, task_t, user_t, int);
 
-#endif /* _GVMD_SCAN_HANDLER_H */
+#endif /* not _GVMD_MANAGE_SCAN_HANDLER_H */
