@@ -12,11 +12,11 @@
 #define _GVMD_MANAGE_SCAN_QUEUE_H
 
 #include <glib.h>
-#include "manage_resources.h"
+#include "manage_resources_types.h"
 
 /**
  * @brief Default maximum number of active scan handlers
- */ 
+ */
 #define DEFAULT_MAX_ACTIVE_SCAN_HANDLERS 3
 
 void
@@ -59,4 +59,4 @@ scan_queue_remove (report_t);
 int
 scan_queue_length ();
 
-#endif /* _GVMD_SCAN_QUEUE_H */
+#endif /* not _GVMD_SCAN_QUEUE_H */
