@@ -18,6 +18,8 @@
 #include "manage_sql_agent_groups.h"
 #include "manage_sql_agents.h"
 #include "manage_sql_copy.h"
+#include "manage_sql_permissions.h"
+#include "manage_sql_resources.h"
 
 #include <util/uuidutils.h>
 
@@ -110,6 +112,53 @@ agent_group_in_use_in_hidden_task (agent_group_t agent_group)
                     agent_group);
 }
 
+/**
+ * @brief Check if an agent group name already exists.
+ *        in agent_groups and agent_groups_trash table.
+ *
+ * @param name The agent group name to check.
+ * @param current_agent_group   The row id of the current agent group or 0.
+ *
+ * @return 1 if the name exists in either agent_groups
+ *         or agent_groups_trash, 0 otherwise.
+ */
+static int
+agent_group_name_exists (const gchar *name, agent_group_t current_agent_group)
+{
+  int count;
+
+  if (current_agent_group)
+    count = sql_int_ps (
+              "SELECT COUNT(*) FROM agent_groups"
+              " WHERE name = $1 AND id != $2;",
+              SQL_STR_PARAM (name),
+              SQL_RESOURCE_PARAM (current_agent_group),
+              NULL);
+  else
+    count = sql_int_ps (
+              "SELECT COUNT(*) FROM agent_groups WHERE name = $1;",
+              SQL_STR_PARAM (name),
+              NULL);
+
+  if (count > 0)
+    return 1;
+
+  if (current_agent_group)
+    count = sql_int_ps (
+              "SELECT COUNT(*) FROM agent_groups_trash"
+              " WHERE name = $1 AND id != $2;",
+              SQL_STR_PARAM (name),
+              SQL_RESOURCE_PARAM (current_agent_group),
+              NULL);
+  else
+    count = sql_int_ps (
+              "SELECT COUNT(*) FROM agent_groups_trash WHERE name = $1;",
+              SQL_STR_PARAM (name),
+              NULL);
+
+  return (count > 0) ? 1 : 0;
+}
+
 /**
  * @brief Count the number of agent groups based on filter criteria.
  *
@@ -124,8 +173,8 @@ agent_group_count (const get_data_t *get)
   static column_t columns[] = AGENT_GROUP_ITERATOR_COLUMNS;
   static column_t trash_columns[] = AGENT_GROUP_ITERATOR_TRASH_COLUMNS;
   const char *join_clause = " LEFT JOIN"
-                            " (SELECT id as scanner_id, name AS scanner_name, uuid AS scanner_uuid FROM scanners )"
-                            " ON scanner_id = scanner";
+                            " (SELECT id as scanner_id, name AS scanner_name, uuid AS scanner_uuid FROM scanners) AS scanner_data"
+                            " ON scanner_data.scanner_id = scanner";
 
   return count ("agent_group", get, columns, trash_columns, extra_columns, 0, join_clause, 0, TRUE);
 }
@@ -145,8 +194,8 @@ init_agent_group_iterator (iterator_t *iterator, get_data_t *get)
   static column_t columns[] = AGENT_GROUP_ITERATOR_COLUMNS;
   static column_t trash_columns[] = AGENT_GROUP_ITERATOR_TRASH_COLUMNS;
   const char *join_clause = " LEFT JOIN"
-                            " (SELECT id as scanner_id, name AS scanner_name, uuid AS scanner_uuid FROM scanners )"
-                            " ON scanner_id = scanner";
+                            " (SELECT id as scanner_id, name AS scanner_name, uuid AS scanner_uuid FROM scanners) AS scanner_data"
+                            " ON scanner_data.scanner_id = scanner";
 
   return init_get_iterator (iterator, "agent_group", get, columns, trash_columns, filter_columns, 0,
                             join_clause, NULL, TRUE);
@@ -213,30 +262,27 @@ create_agent_group (agent_group_data_t group_data,
         return AGENT_GROUP_RESP_INTERNAL_ERROR;
     }
 
-  gchar *quoted_uuid = sql_quote (group_data->uuid);
-  gchar *quoted_name = sql_quote (group_data->name);
-  gchar *quoted_comment = sql_quote (group_data->comment);
-  gchar *quoted_user_uuid = sql_quote (current_credentials.uuid);
+  if (agent_group_name_exists (group_data->name, 0))
+    {
+      g_debug ("%s: agent group name already exists", __func__);
+      return AGENT_GROUP_RESP_GROUP_NAME_EXISTS;
+    }
 
   sql_begin_immediate ();
 
   // Insert into agent_groups (scanner added)
-  sql ("INSERT INTO agent_groups (uuid, name, comment, scanner, owner, creation_time, modification_time) "
-       "VALUES ('%s', '%s', '%s', %llu, "
-       "  (SELECT id FROM users WHERE uuid = '%s'),"
-       "  %ld, %ld);",
-       quoted_uuid,
-       quoted_name,
-       quoted_comment,
-       group_data->scanner,
-       quoted_user_uuid,
-       group_data->creation_time,
-       group_data->modification_time);
-
-  g_free (quoted_uuid);
-  g_free (quoted_name);
-  g_free (quoted_comment);
-  g_free (quoted_user_uuid);
+  sql_ps ("INSERT INTO agent_groups (uuid, name, comment, scanner, owner, creation_time, modification_time) "
+       "VALUES ($1, $2, $3, $4, "
+       "  (SELECT id FROM users WHERE uuid = $5),"
+       "  $6, $7);",
+       SQL_STR_PARAM (group_data->uuid),
+       SQL_STR_PARAM (group_data->name),
+       SQL_STR_PARAM (group_data->comment),
+       SQL_RESOURCE_PARAM (group_data->scanner),
+       SQL_STR_PARAM (current_credentials.uuid),
+       SQL_INT_PARAM (group_data->creation_time),
+       SQL_INT_PARAM (group_data->modification_time),
+       NULL);
 
   agent_group_t new_agent_group = sql_last_insert_id ();
   if (new_agent_group == 0)
@@ -265,6 +311,11 @@ create_agent_group (agent_group_data_t group_data,
            else if (result == 2)
              return AGENT_GROUP_RESP_AGENT_SCANNER_MISMATCH;
         }
+      /* Verify that the agent is authorized before adding it to the group. */
+      if (!agent_authorized (uuid, group_data->scanner))
+        {
+          return AGENT_GROUP_RESP_AGENT_UNAUTHORIZED;
+        }
 
       db_copy_buffer_append_printf (&buffer, "%llu\t%llu\n", new_agent_group, agent_id);
     }
@@ -293,20 +344,21 @@ modify_agent_group (agent_group_t agent_group,
                     agent_group_data_t group_data,
                     agent_uuid_list_t agent_uuids)
 {
-  gchar *quoted_name = sql_quote (group_data->name);
-  gchar *quoted_comment = sql_quote (group_data->comment);
 
+  if (agent_group_name_exists (group_data->name, agent_group))
+    {
+      g_debug ("%s: agent group name already exists", __func__);
+      return AGENT_GROUP_RESP_GROUP_NAME_EXISTS;
+    }
   sql_begin_immediate ();
 
-  sql ("UPDATE agent_groups SET name = '%s', comment = '%s', "
-       "modification_time = %ld WHERE id = %llu;",
-       quoted_name,
-       quoted_comment,
-       group_data->modification_time,
-       agent_group);
-
-  g_free (quoted_name);
-  g_free (quoted_comment);
+  sql_ps ("UPDATE agent_groups SET name = $1, comment = $2, "
+       "modification_time = $3 WHERE id = $4;",
+       SQL_STR_PARAM (group_data->name),
+       SQL_STR_PARAM (group_data->comment),
+       SQL_INT_PARAM (group_data->modification_time),
+       SQL_RESOURCE_PARAM (agent_group),
+       NULL);
 
   if (!agent_uuids || agent_uuids->count == 0)
     {
@@ -341,21 +393,28 @@ modify_agent_group (agent_group_t agent_group,
 
   for (int i = 0; i < agent_uuids->count; ++i)
     {
-    const gchar *uuid = agent_uuids->agent_uuids[i];
-    agent_t agent_id;
-    int result = agent_id_by_uuid_and_scanner (uuid, group_data->scanner, &agent_id);
-    if (result != 0)
-    {
-      db_copy_buffer_cleanup (&buffer);
-      sql_rollback ();
+      const gchar *uuid = agent_uuids->agent_uuids[i];
+      agent_t agent_id;
+      int result = agent_id_by_uuid_and_scanner (
+        uuid, group_data->scanner, &agent_id);
+      if (result != 0)
+        {
+          db_copy_buffer_cleanup (&buffer);
+          sql_rollback ();
 
-      if (result == 1)
-        return AGENT_GROUP_RESP_AGENT_NOT_FOUND;
-      else if (result == 2)
-        return AGENT_GROUP_RESP_AGENT_SCANNER_MISMATCH;
-    }
+          if (result == 1)
+            return AGENT_GROUP_RESP_AGENT_NOT_FOUND;
+          else if (result == 2)
+            return AGENT_GROUP_RESP_AGENT_SCANNER_MISMATCH;
+        }
 
-     db_copy_buffer_append_printf (&buffer, "%llu\t%llu\n", agent_group, agent_id);
+      /* Verify that the agent is authorized before adding it to the group. */
+      if (!agent_authorized (uuid, group_data->scanner))
+        {
+          return AGENT_GROUP_RESP_AGENT_UNAUTHORIZED;
+        }
+      db_copy_buffer_append_printf (&buffer, "%llu\t%llu\n", agent_group,
+                                    agent_id);
     }
 
   db_copy_buffer_commit (&buffer, TRUE);
@@ -690,6 +749,35 @@ agent_group_uuid (agent_group_t group_id)
   return sql_string ("SELECT uuid FROM agent_groups WHERE id = %llu;", group_id);
 }
 
+
+/**
+ * @brief Return the name of an agent group.
+ *
+ * @param[in]  group_id  Agent group ID.
+ *
+ * @return Newly allocated name  if found, else NULL.
+ */
+char *
+agent_group_name (agent_group_t group_id)
+{
+  g_return_val_if_fail (group_id, NULL);
+  return sql_string ("SELECT name FROM agent_groups WHERE id = %llu;", group_id);
+}
+
+/**
+ * @brief Return the comment of an agent group.
+ *
+ * @param[in]  group_id  Agent group ID.
+ *
+ * @return Newly allocated comment  if found, else NULL.
+ */
+char *
+agent_group_comment (agent_group_t group_id)
+{
+  g_return_val_if_fail (group_id, NULL);
+  return sql_string ("SELECT comment FROM agent_groups WHERE id = %llu;", group_id);
+}
+
 /**
  * @brief Return the row_id of an agent group.
  *
@@ -831,6 +919,54 @@ trash_agent_group_in_use (agent_group_t agent_group)
                     agent_group);
 }
 
+/**
+ * @brief Return whether an agent_group is readable.
+ *
+ * @param[in]  agent_group  Row id in agent_groups table.
+ *
+ * @return 1 if readable, 0 otherwise.
+ */
+int
+agent_group_readable (agent_group_t agent_group)
+{
+  char *uuid;
+  agent_group_t found = 0;
+
+  if (agent_group == 0)
+    return 0;
+  uuid = agent_group_uuid (agent_group);
+  if (uuid == NULL)
+    return 0;
+  find_agent_group_with_permission (uuid, &found, "get_agent_groups");
+  g_free (uuid);
+  return found > 0;;
+}
+
+/**
+ * @brief Return whether a trashcan agent_group is readable.
+ *
+ * @param[in]  agent_group  Row id in agent_groups_trash.
+ *
+ * @return 1 if readable, 0 otherwise.
+ */
+int
+trash_agent_group_readable (agent_group_t agent_group)
+{
+  char *uuid;
+  agent_group_t found = 0;
+
+  if (agent_group == 0)
+    return 0;
+  uuid = trash_agent_group_uuid (agent_group);
+  if (find_trash ("agent_group", uuid, &found))
+    {
+      g_free (uuid);
+      return 0;
+    }
+  g_free (uuid);
+  return found > 0;
+}
+
 /**
  * @brief Return whether a agent_group is writable.
  *
@@ -871,4 +1007,55 @@ agent_group_scanner (agent_group_t agent_group)
                       agent_group);
 }
 
+/**
+ * @brief Return the UUID of a trashed agent group.
+ *
+ * @param[in]  agent_group  Row id in agent_groups_trash.
+ *
+ * @return Newly allocated string (caller must g_free) or NULL if not found.
+ */
+char *
+trash_agent_group_uuid (agent_group_t agent_group)
+{
+  if (!agent_group)
+    return NULL;
+
+  return sql_string ("SELECT uuid FROM agent_groups_trash WHERE id = %llu;",
+                     agent_group);
+}
+
+/**
+ * @brief Return the name of a trashed agent group.
+ *
+ * @param[in]  agent_group  Row id in agent_groups_trash.
+ *
+ * @return Newly allocated string (caller must g_free) or NULL if not found.
+ */
+char *
+trash_agent_group_name (agent_group_t agent_group)
+{
+  if (!agent_group)
+    return NULL;
+
+  return sql_string ("SELECT name FROM agent_groups_trash WHERE id = %llu;",
+                     agent_group);
+}
+
+/**
+ * @brief Return the comment of a trashed agent group.
+ *
+ * @param[in]  agent_group  Row id in agent_groups_trash.
+ *
+ * @return Newly allocated comment (caller must g_free) or NULL if not found.
+ */
+char *
+trash_agent_group_comment (agent_group_t agent_group)
+{
+  if (!agent_group)
+    return NULL;
+
+  return sql_string ("SELECT comment FROM agent_groups_trash WHERE id = %llu;",
+                     agent_group);
+}
+
 #endif // ENABLE_AGENTS
