@@ -1,25 +1,13 @@
 /* Copyright (C) 2009-2025 Greenbone AG
  *
  * SPDX-License-Identifier: AGPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _GVMD_MANAGE_SQL_ASSETS_H
 #define _GVMD_MANAGE_SQL_ASSETS_H
 
-#include "manage_resources.h"
+#include "manage_assets.h"
+#include "manage_resources_types.h"
 
 /**
  * @file
@@ -275,6 +263,21 @@ result_host_asset_id (const char *, result_t);
 int
 manage_report_host_detail (report_t, const char *, const char *, GHashTable *);
 
+char *
+report_host_ip (const char *);
+
+gchar *
+report_host_hostname (report_host_t);
+
+gchar *
+report_host_best_os_cpe (report_host_t);
+
+gchar *
+report_host_best_os_txt (report_host_t);
+
+int
+report_host_noticeable (report_t, const gchar *);
+
 gchar*
 asset_host_extra_where (const char *);
 
