@@ -7,8 +7,9 @@
 
 #include "debug_utils.h"
 #include "manage_sql_oci_image_targets.h"
-#include "manage_oci_image_targets.h"
 #include "manage_acl.h"
+#include "manage_sql_permissions.h"
+#include "manage_sql_resources.h"
 #include "sql.h"
 #include "utils.h"
 
@@ -25,22 +26,21 @@
  * @param[in]   comment          Comment on target.
  * @param[in]   credential_id    Credential for accessing the registry.
  * @param[in]   image_references Image OCI URLs
+ * @param[in]   exclude_images   Excluded image OCI URLs
  * @param[out]  oci_image_target Created target.
  * @param[out]  error_message    Error message if any.
  *
- * @return 0 success, 1 target exists already, 2 error in image URLS,
- *         3 invalid credential, 4 could not find credential,
- *         5 invalid credential type, 99 permission denied, -1 error.
+ * @return A member of create_oci_image_target_return_t.
  */
-int
+create_oci_image_target_return_t
 create_oci_image_target (const char* name,
                          const char* comment,
                          const char* image_references,
+                         const char* exclude_images,
                          const char *credential_id,
                          oci_image_target_t* oci_image_target,
                          gchar **error_message)
 {
-  gchar *quoted_name, *quoted_urls, *quoted_comment;
   oci_image_target_t new_oci_image_target;
   credential_t credential = 0;
 
@@ -51,30 +51,37 @@ create_oci_image_target (const char* name,
   if (acl_user_may ("create_oci_image_target") == 0)
     {
       sql_rollback ();
-      return 99;
+      return CREATE_OCI_IMAGE_TARGET_PERMISSION_DENIED;
     }
 
   if (resource_with_name_exists (name, "oci_image_target", 0))
     {
       sql_rollback ();
-      return 1;
+      return CREATE_OCI_IMAGE_TARGET_EXISTS_ALREADY;
     }
-  
-  quoted_name = sql_quote (name ?: "");
 
-  if (!validate_oci_image_references (image_references, error_message))
+  gchar *clean_references = clean_images (image_references);
+  if (!clean_references
+      || !validate_oci_image_references (clean_references, error_message))
     {
       sql_rollback ();
-      g_free (quoted_name);
-      return 2;
+      g_free (clean_references);
+      return CREATE_OCI_IMAGE_TARGET_INVALID_IMAGE_URLS;
     }
 
-  quoted_urls = sql_quote (image_references);
-
-  if (comment)
-    quoted_comment = sql_quote (comment);
-  else
-    quoted_comment = sql_quote ("");
+  gchar *clean_excludes = NULL;
+  if (exclude_images && strlen (exclude_images) > 0)
+    {
+      clean_excludes = clean_images (exclude_images);
+      if (!clean_excludes || !validate_oci_image_references (clean_excludes,
+                                                             error_message))
+        {
+          sql_rollback ();
+          g_free (clean_references);
+          g_free (clean_excludes);
+          return CREATE_OCI_IMAGE_TARGET_INVALID_EXCLUDE_IMAGES;
+        }
+    }
 
   if (credential_id)
     {
@@ -86,70 +93,62 @@ create_oci_image_target (const char* name,
                                                "get_credentials"))
             {
               sql_rollback ();
-              g_free (quoted_name);
-              g_free (quoted_urls);
-              g_free (quoted_comment);
-              return -1;
+              return CREATE_OCI_IMAGE_TARGET_INTERNAL_ERROR;
             }
 
           if (credential == 0)
             {
               sql_rollback ();
-              g_free (quoted_name);
-              g_free (quoted_urls);
-              g_free (quoted_comment);
-              return 4;
+              return CREATE_OCI_IMAGE_TARGET_CREDENTIAL_NOT_FOUND;
             }
 
           type = credential_type (credential);
           if (strcmp (type, "up"))
             {
               sql_rollback ();
-              g_free (quoted_name);
-              g_free (quoted_urls);
-              g_free (quoted_comment);
               g_free (type);
-              return 5;
+              return CREATE_OCI_IMAGE_TARGET_INVALID_CREDENTIAL_TYPE;
             }
           g_free (type);
         }
       else
         {
           sql_rollback ();
-          g_free (quoted_name);
-          g_free (quoted_urls);
-          g_free (quoted_comment);
-          return 3;
+          return CREATE_OCI_IMAGE_TARGET_INVALID_CREDENTIAL;
         }
     }
 
-  sql ("INSERT INTO oci_image_targets"
-       " (uuid, name, owner, image_references, comment,"
-       "  creation_time, modification_time)"
-       " VALUES (make_uuid (), '%s',"
-       " (SELECT id FROM users WHERE users.uuid = '%s'),"
-       " '%s', '%s', m_now (), m_now ());",
-        quoted_name, current_credentials.uuid,
-        quoted_urls, quoted_comment);
+  sql_ps ("INSERT INTO oci_image_targets"
+          " (uuid, name, owner, image_references, exclude_images,"
+          "  comment, creation_time, modification_time)"
+          " VALUES (make_uuid (), $1,"
+          " (SELECT id FROM users WHERE users.uuid = $2),"
+          " $3, $4, $5, m_now (), m_now ());",
+          SQL_STR_PARAM (name),
+          SQL_STR_PARAM (current_credentials.uuid),
+          SQL_STR_PARAM (clean_references),
+          clean_excludes ? SQL_STR_PARAM (clean_excludes) : SQL_NULL_PARAM,
+          comment ? SQL_STR_PARAM (comment) : SQL_NULL_PARAM,
+          NULL);
 
   new_oci_image_target = sql_last_insert_id ();
 
   if (credential)
-    sql ("UPDATE oci_image_targets SET credential = %llu"
-         " WHERE id = %llu;",
-         credential,
-         new_oci_image_target);
+    sql_ps ("UPDATE oci_image_targets SET credential = $1"
+            " WHERE id = $2;",
+            SQL_RESOURCE_PARAM (credential),
+            SQL_RESOURCE_PARAM (new_oci_image_target),
+            NULL);
 
   if (oci_image_target)
     *oci_image_target = new_oci_image_target;
 
-  g_free (quoted_comment);
-  g_free (quoted_name);
-  g_free (quoted_urls);
-
   sql_commit ();
 
-  return 0;
+  g_free (clean_references);
+  g_free (clean_excludes);
+
+  return CREATE_OCI_IMAGE_TARGET_OK;
 }
 
 /**
@@ -175,8 +174,8 @@ copy_oci_image_target (const char* name,
   assert (new_oci_image_target);
 
   ret = copy_resource ("oci_image_target", name, comment, oci_image_target_id,
-                       "credential, image_references", 1, new_oci_image_target,
-                       &old_oci_image_target);
+                       "credential, image_references, exclude_images", 1,
+                       new_oci_image_target, &old_oci_image_target);
   if (ret)
     return ret;
 
@@ -191,18 +190,16 @@ copy_oci_image_target (const char* name,
  * @param[in]   comment              Comment on target.
  * @param[in]   credential_id        Credential.
  * @param[in]   image_references     List of image urls.
+ * @param[in]   exclude_images       List of excluded image urls.
  * @param[out]  error_message        Error message if any.
  *
- * @return 0 success, 1 failed to find target, 
- *         2 zero length name, 3 target exists already,
- *         4 target is in use, 5 failed to find credential,
- *         6 invalid credential type, 7 error in image urls
- *         specification, 99 permission denied, -1 error.
+ * @return A member of modify_oci_image_target_return_t.
  */
-int
+modify_oci_image_target_return_t
 modify_oci_image_target (const char *oci_image_target_id, const char *name,
                          const char *comment, const char *credential_id,
-                         const char *image_references, gchar **error_message)
+                         const char *image_references, const char *exclude_images,
+                         gchar **error_message)
 {
   oci_image_target_t oci_image_target;
   credential_t credential;
@@ -215,7 +212,7 @@ modify_oci_image_target (const char *oci_image_target_id, const char *name,
   if (acl_user_may ("modify_oci_image_target") == 0)
     {
       sql_rollback ();
-      return 99;
+      return MODIFY_OCI_IMAGE_TARGET_PERMISSION_DENIED;
     }
 
   oci_image_target = 0;
@@ -224,55 +221,46 @@ modify_oci_image_target (const char *oci_image_target_id, const char *name,
                                              "modify_oci_image_target"))
     {
       sql_rollback ();
-      return -1;
+      return MODIFY_OCI_IMAGE_TARGET_INTERNAL_ERROR;
     }
 
   if (oci_image_target == 0)
     {
       sql_rollback ();
-      return 1;
+      return MODIFY_OCI_IMAGE_TARGET_NOT_FOUND;
     }
 
   if (name)
     {
-      gchar *quoted_name;
-
       if (strlen (name) == 0)
         {
           sql_rollback ();
-          return 2;
+          return MODIFY_OCI_IMAGE_TARGET_INVALID_NAME;
         }
       if (resource_with_name_exists (name, "oci_image_target", oci_image_target))
         {
           sql_rollback ();
-          return 3;
+          return MODIFY_OCI_IMAGE_TARGET_EXISTS_ALREADY;
         }
 
-      quoted_name = sql_quote (name);
-
-      sql ("UPDATE oci_image_targets SET"
-           " name = '%s',"
-           " modification_time = m_now ()"
-           " WHERE id = %llu;",
-           quoted_name,
-           oci_image_target);
-
-      g_free (quoted_name);
+      sql_ps ("UPDATE oci_image_targets SET"
+              " name = $1,"
+              " modification_time = m_now ()"
+              " WHERE id = $2;",
+              SQL_STR_PARAM (name),
+              SQL_RESOURCE_PARAM (oci_image_target),
+              NULL);
     }
 
   if (comment)
     {
-      gchar *quoted_comment;
-      quoted_comment = sql_quote (comment);
-
-      sql ("UPDATE oci_image_targets SET"
-           " comment = '%s',"
-           " modification_time = m_now ()"
-           " WHERE id = %llu;",
-           quoted_comment,
-           oci_image_target);
-
-      g_free (quoted_comment);
+      sql_ps ("UPDATE oci_image_targets SET"
+              " comment = $1,"
+              " modification_time = m_now ()"
+              " WHERE id = $2;",
+              SQL_STR_PARAM (comment),
+              SQL_RESOURCE_PARAM (oci_image_target),
+              NULL);
     }
 
   if (credential_id)
@@ -280,7 +268,7 @@ modify_oci_image_target (const char *oci_image_target_id, const char *name,
       if (oci_image_target_in_use (oci_image_target))
         {
           sql_rollback ();
-          return 4;
+          return MODIFY_OCI_IMAGE_TARGET_IN_USE;
         }
 
       credential = 0;
@@ -292,63 +280,96 @@ modify_oci_image_target (const char *oci_image_target_id, const char *name,
                                                "get_credentials"))
             {
               sql_rollback ();
-              return -1;
+              return MODIFY_OCI_IMAGE_TARGET_INTERNAL_ERROR;
             }
 
           if (credential == 0)
             {
               sql_rollback ();
-              return 5;
+              return MODIFY_OCI_IMAGE_TARGET_CREDENTIAL_NOT_FOUND;
             }
 
           type = credential_type (credential);
           if (strcmp (type, "up"))
             {
               sql_rollback ();
-              return 6;
+              return MODIFY_OCI_IMAGE_TARGET_INVALID_CREDENTIAL_TYPE;
             }
           g_free (type);
 
-          sql ("UPDATE oci_image_targets SET"
-              " credential = %llu,"
-              " modification_time = m_now ()"
-              " WHERE id = %llu;",
-              credential,
-              oci_image_target);
+          sql_ps ("UPDATE oci_image_targets SET"
+                   " credential = $1,"
+                   " modification_time = m_now ()"
+                   " WHERE id = $2;",
+                   SQL_RESOURCE_PARAM (credential),
+                   SQL_RESOURCE_PARAM (oci_image_target),
+                   NULL);
         }
       else
-        sql ("UPDATE oci_image_targets SET"
-             " credential = NULL,"
-             " modification_time = m_now ()"
-             " WHERE id = %llu;",
-             oci_image_target);
+        sql_ps ("UPDATE oci_image_targets SET"
+                " credential = NULL,"
+                " modification_time = m_now ()"
+                " WHERE id = $1;",
+                SQL_RESOURCE_PARAM (oci_image_target),
+                NULL);
     }
 
   if (image_references)
     {
-      gchar *quoted_image_references;
-
-      if (!validate_oci_image_references (image_references, error_message))
+      gchar *clean_references = clean_images (image_references);
+      if (!clean_references
+          || !validate_oci_image_references (clean_references, error_message))
         {
           sql_rollback ();
-          return 7;
+          g_free (clean_references);
+          return MODIFY_OCI_IMAGE_TARGET_INVALID_IMAGE_URLS;
         }
 
-      quoted_image_references = sql_quote (image_references);
+      sql_ps ("UPDATE oci_image_targets SET"
+              " image_references = $1,"
+              " modification_time = m_now ()"
+              " WHERE id = $2;",
+              SQL_STR_PARAM (clean_references),
+              SQL_RESOURCE_PARAM (oci_image_target),
+              NULL);
+      g_free (clean_references);
+    }
 
-      sql ("UPDATE oci_image_targets SET"
-           " image_references = '%s',"
-           " modification_time = m_now ()"
-           " WHERE id = %llu;",
-           quoted_image_references,
-           oci_image_target);
+  if (exclude_images)
+    {
+      if (g_str_equal (exclude_images, ""))
+        {
+          sql_ps ("UPDATE oci_image_targets SET"
+                  " exclude_images = NULL,"
+                  " modification_time = m_now ()"
+                  " WHERE id = $1;",
+                  SQL_RESOURCE_PARAM (oci_image_target),
+                  NULL);
+        }
+      else
+        {
+          gchar *clean_excludes = clean_images (exclude_images);
+          if (!clean_excludes || !validate_oci_image_references (clean_excludes, error_message))
+            {
+              sql_rollback ();
+              g_free (clean_excludes);
+              return MODIFY_OCI_IMAGE_TARGET_INVALID_EXCLUDE_IMAGES;
+            }
 
-      g_free (quoted_image_references);
+          sql_ps ("UPDATE oci_image_targets SET"
+                  " exclude_images = $1,"
+                  " modification_time = m_now ()"
+                  " WHERE id = $2;",
+                  SQL_STR_PARAM (clean_excludes),
+                  SQL_RESOURCE_PARAM (oci_image_target),
+                  NULL);
+          g_free (clean_excludes);
+        }
     }
 
   sql_commit ();
 
-  return 0;
+  return MODIFY_OCI_IMAGE_TARGET_OK;
 }
 
 /**
@@ -404,7 +425,7 @@ delete_oci_image_target (const char *oci_image_target_id, int ultimate)
       /* Check if it's in use by a task in the trashcan. */
       if (sql_int ("SELECT count(*) FROM tasks"
                    " WHERE oci_image_target = %llu"
-                   " AND oci_image_target_location = " 
+                   " AND oci_image_target_location = "
                    G_STRINGIFY (LOCATION_TRASH) ";",
                    oci_image_target))
         {
@@ -428,7 +449,7 @@ delete_oci_image_target (const char *oci_image_target_id, int ultimate)
     {
       if (sql_int ("SELECT count(*) FROM tasks"
                    " WHERE oci_image_target = %llu"
-                   " AND oci_image_target_location = " 
+                   " AND oci_image_target_location = "
                    G_STRINGIFY (LOCATION_TABLE)
                    " AND hidden = 0;",
                    oci_image_target))
@@ -438,11 +459,11 @@ delete_oci_image_target (const char *oci_image_target_id, int ultimate)
         }
 
       sql ("INSERT INTO oci_image_targets_trash"
-           " (uuid, owner, name, image_references, comment,"
-           "  credential, credential_location, creation_time,"
+           " (uuid, owner, name, image_references, exclude_images,"
+           "  comment, credential, credential_location, creation_time,"
            "  modification_time)"
-           " SELECT uuid, owner, name, image_references, comment,"
-           " credential, " G_STRINGIFY (LOCATION_TABLE) ","
+           " SELECT uuid, owner, name, image_references, exclude_images,"
+           " comment, credential, " G_STRINGIFY (LOCATION_TABLE) ","
            " creation_time, modification_time"
            " FROM oci_image_targets WHERE id = %llu;",
            oci_image_target);
@@ -450,10 +471,10 @@ delete_oci_image_target (const char *oci_image_target_id, int ultimate)
       /* Update the location of the target in any task. */
       sql ("UPDATE tasks"
            " SET oci_image_target = %llu,"
-           "     oci_image_target_location = " 
+           "     oci_image_target_location = "
            G_STRINGIFY (LOCATION_TRASH)
            " WHERE oci_image_target = %llu"
-           " AND oci_image_target_location = " 
+           " AND oci_image_target_location = "
            G_STRINGIFY (LOCATION_TABLE) ";",
            sql_last_insert_id (),
            oci_image_target);
@@ -545,10 +566,10 @@ restore_oci_image_target (const char *oci_image_target_id)
   /* Move to "real" tables. */
   sql ("INSERT INTO oci_image_targets"
        " (uuid, owner, name, comment, creation_time, modification_time,"
-       "  image_references, credential)"
+       "  image_references, exclude_images, credential)"
        " SELECT"
        "  uuid, owner, name, comment, creation_time, modification_time,"
-       "  image_references, credential"
+       "  image_references, exclude_images, credential"
        " FROM oci_image_targets_trash"
        " WHERE id = %llu;",
        resource);
@@ -582,7 +603,8 @@ restore_oci_image_target (const char *oci_image_target_id)
  * @brief Filter columns for oci image target iterator.
  */
 #define OCI_IMAGE_TARGET_ITERATOR_FILTER_COLUMNS               \
- { GET_ITERATOR_FILTER_COLUMNS, "image_references", "credential_name", NULL }
+ { GET_ITERATOR_FILTER_COLUMNS, "image_references", "exclude_images", \
+  "credential_name", NULL }
 
 /**
  * @brief OCI Image Target iterator columns.
@@ -591,6 +613,7 @@ restore_oci_image_target (const char *oci_image_target_id)
  {                                                             \
    GET_ITERATOR_COLUMNS (oci_image_targets),                   \
    { "image_references", NULL, KEYWORD_TYPE_STRING },          \
+   { "exclude_images", NULL, KEYWORD_TYPE_STRING },            \
    { "credential", NULL, KEYWORD_TYPE_INTEGER },               \
    {                                                           \
      "(SELECT name FROM credentials WHERE id = credential)",   \
@@ -608,6 +631,7 @@ restore_oci_image_target (const char *oci_image_target_id)
  {                                                                      \
    GET_ITERATOR_COLUMNS (oci_image_targets_trash),                      \
    { "image_references", NULL, KEYWORD_TYPE_STRING },                   \
+   { "exclude_images", NULL, KEYWORD_TYPE_STRING },                     \
    { "credential", NULL, KEYWORD_TYPE_INTEGER },                        \
    {                                                                    \
      "(SELECT CASE"                                                     \
@@ -676,6 +700,16 @@ init_oci_image_target_iterator (iterator_t* iterator, get_data_t *get)
  */
 DEF_ACCESS (oci_image_target_iterator_image_refs, GET_ITERATOR_COLUMN_COUNT);
 
+/**
+ * @brief Get the excluded images of from an oci image target iterator.
+ *
+ * @param[in]  iterator  Iterator.
+ *
+ * @return Excluded images of the target or NULL if iteration is complete.
+ */
+DEF_ACCESS (oci_image_target_iterator_exclude_images,
+              GET_ITERATOR_COLUMN_COUNT + 1);
+
 /**
  * @brief Get the credential from an oci image target iterator.
  *
@@ -688,7 +722,7 @@ oci_image_target_iterator_credential (iterator_t* iterator)
 {
   if (iterator->done)
     return 0;
-  return iterator_int64 (iterator, GET_ITERATOR_COLUMN_COUNT + 1);
+  return iterator_int64 (iterator, GET_ITERATOR_COLUMN_COUNT + 2);
 }
 
 /**
@@ -699,7 +733,8 @@ oci_image_target_iterator_credential (iterator_t* iterator)
  * @return Credential name, or 0 if iteration is complete. Freed by
  *         cleanup_iterator.
  */
-DEF_ACCESS (oci_image_target_iterator_credential_name, GET_ITERATOR_COLUMN_COUNT + 2);
+DEF_ACCESS (oci_image_target_iterator_credential_name,
+             GET_ITERATOR_COLUMN_COUNT + 3);
 
 /**
  * @brief Get the credential location of the oci image target from iterator.
@@ -713,7 +748,7 @@ oci_image_target_iterator_credential_trash (iterator_t *iterator)
 {
   if (iterator->done)
     return 0;
-  return iterator_int (iterator, GET_ITERATOR_COLUMN_COUNT + 3);
+  return iterator_int (iterator, GET_ITERATOR_COLUMN_COUNT + 4);
 }
 
 /**
@@ -819,6 +854,22 @@ oci_image_target_image_references (oci_image_target_t oci_image_target)
                      " WHERE id = %llu;", oci_image_target);
 }
 
+/**
+ * @brief Return the excluded images of an OCI image target.
+ *
+ * @param[in]  oci_image_target  OCI Image Target.
+ *
+ * @return Newly allocated comma separated list of excluded images
+ *         if available, else NULL.
+ */
+char*
+oci_image_target_exclude_images (oci_image_target_t oci_image_target)
+{
+  return sql_string_ps ("SELECT exclude_images FROM oci_image_targets"
+                        " WHERE id = $1;",
+                        SQL_RESOURCE_PARAM (oci_image_target), NULL);
+}
+
 /**
  * @brief Return whether a trashcan oci_image_target is readable.
  *
@@ -856,7 +907,7 @@ oci_image_target_in_use (oci_image_target_t oci_image_target)
 {
   return !!sql_int ("SELECT count(*) FROM tasks"
                     " WHERE oci_image_target = %llu"
-                    " AND oci_image_target_location = " 
+                    " AND oci_image_target_location = "
                     G_STRINGIFY (LOCATION_TABLE)
                     " AND hidden = 0;",
                     oci_image_target);
@@ -874,7 +925,7 @@ trash_oci_image_target_in_use (oci_image_target_t oci_image_target)
 {
   return !!sql_int ("SELECT count(*) FROM tasks"
                     " WHERE oci_image_target = %llu"
-                    " AND oci_image_target_location = " 
+                    " AND oci_image_target_location = "
                     G_STRINGIFY (LOCATION_TRASH),
                     oci_image_target);
 }
