@@ -1,19 +1,6 @@
 /* Copyright (C) 2020-2022 Greenbone AG
  *
  * SPDX-License-Identifier: AGPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -25,7 +12,8 @@
 
 #include "manage_sql_port_lists.h"
 #include "manage_acl.h"
-#include "manage_port_lists.h"
+#include "manage_sql_permissions.h"
+#include "manage_sql_resources.h"
 #include "sql.h"
 
 #include <errno.h>
@@ -859,21 +847,6 @@ port_list_predefined (port_list_t port_list)
                   port_list);
 }
 
-/**
- * @brief Return whether a trash port list is predefined.
- *
- * @param[in]  port_list  Port list.
- *
- * @return 1 if predefined, else 0.
- */
-int
-trash_port_list_predefined (port_list_t port_list)
-{
-  return sql_int ("SELECT predefined FROM port_lists_trash"
-                  " WHERE id = %llu;",
-                  port_list);
-}
-
 /**
  * @brief Return the UUID of the port list of a port_range.
  *
@@ -2192,7 +2165,7 @@ init_port_range_iterator (iterator_t* iterator, port_list_t port_list,
     }
   else
     init_iterator (iterator,
-                   "SELECT uuid, comment, start, end, type, exclude"
+                   "SELECT uuid, comment, start, \"end\", type, exclude"
                    " FROM port_ranges%s"
                    " WHERE"
                    " (((SELECT owner FROM port_lists%s WHERE id = port_list)"
