@@ -1,33 +1,21 @@
 /* Copyright (C) 2024 Greenbone AG
  *
  * SPDX-License-Identifier: AGPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
- * @file manage_sql_report_configs.c
+ * @file
  * @brief GVM management layer: Report configs SQL.
  *
  * SQL report config code for the GVM management layer.
  */
 
 #include "debug_utils.h"
-#include "manage_report_configs.h"
 #include "manage_sql_report_configs.h"
 #include "manage_acl.h"
-#include "manage_report_formats.h"
+#include "manage_sql_permissions.h"
+#include "manage_sql_report_formats.h"
+#include "manage_sql_resources.h"
 #include "sql.h"
 #include "utils.h"
 #include <glib.h>
