@@ -1,19 +1,6 @@
 /* Copyright (C) 2019-2022 Greenbone AG
  *
  * SPDX-License-Identifier: AGPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -23,11 +10,11 @@
  * The TLS Certificates SQL for the GVM management layer.
  */
 
-#include "manage_tls_certificates.h"
-#include "manage_acl.h"
-#include "manage_filters.h"
 #include "manage_sql_tls_certificates.h"
+#include "manage_acl.h"
 #include "manage_sql.h"
+#include "manage_sql_filters.h"
+#include "manage_sql_resources.h"
 #include "utils.h"
 #include "sql.h"
 
